; Dtecteur de passage par rflexion  base de PIC 12F675.   Le 28/04/2021
;
;
; Ce programme est libre de droit. 
; Si il est utilis dans des montages personnels ou commerciaux, il faut que le fichier source original ".asm' et les sources modifis soient livrs avec le montage.
; 
;                            12F675
;          +5 Volts    VDD  [1    8]  VSS  Masse
;  Sortie   Output     GP5  [2    7]  GP0 AN0 = Potentiomtre de rglage de la sensibilit de la dtection
;  LED IR   Output     GP4  [3    6]  GP1 AN1 = Potentiomtre de rglage de la dure du signal en sortie
;  Refl/Dir Input      GP3  [4    5]  GP2 AN2 = Entre du phototransistor
; 
; Entre analogique GP0 = AN0 = 0-5 Volts => Rglage de la sensibilit de la dtection optique, 
;   agit sur la valeur de seuil de comparaison led allume/led teinte.
; La plage de sensibilit est dcoupe en 16 pas : (0 - 255)/16.
;          1 = trs sensible  44 = peu sensible.
;     0 :  1   |    1 :  2   |    2 :  3   |    3 :  4   |    4 :  5   |
;     5 :  6   |    6 :  8   |    7 : 10   |    8 : 13   |    9 : 16   |
;    10 : 19   |   11 : 23   |   12 : 27   |   13 : 32   |   14 : 37   |   15 : 44
; 
; Entre analogique GP1 = AN1 = 0-5 Volts = Rglage de la dure du signal en sortie, de 0  25 secondes.
; Avec une dure de dcomptage de boucle = 0,1 sec => Temporisation = 0  25 secondes en sortie.
; Si un train est detect, la temporisation est rarme.
; 
; Sortie GP4 = Sortie impulsionnelle de 1,5 msec pour alimenter la LED infrarouge.
; Sortie courant max du PIC = 20 mA par patte.
; Dure minimum de 1,5 msec, car le phototransistor est trs lent  ragir.
; 
; Sortie GP5 = Sortie du montage temporise vers la base d'un transistor NPN = BC847C ou BC846C.
; La sortie est  collecteur ouvert.
; 
; Entre analogique GP2 = AN2 = 0-5 Volts => Entre du signal issu du phototransistor.
; 
; +5V ---+----------------+-------------                     Sensibilit                 Temporisation
;        <  100 Kohms      < 100 Kohms                            _____                        _____             
;        >                >                                    /  |  \                      /   / \            
;        +----> GP0       +----> GP1                    moins  |  O  |  plus           0    |  O  |  25
;        <  Sensibilit   <  Temporisation            sensible |__ __| sensible     seconde |__ __| secondes   
;        >                >                                    V  V  V                      V  V  V            
; 0V  ---+----------------+-------------
; 
; GP4 ----/\/\/\/\/---------<|------ +5V 
;       240 Ohms=14mA     LED-IR          
; 
; +5V -------------------+--------
;                        < 10 KOhms  220 KOhms
;                        >
;                        +----> GP2
;                       /
;                      |   TR-IR
;                       \
;                        |
; 0V  -------------------+--------
; 
; 
;                        +--<|----> OUT
;           1,5 Kohms    /
;  GP5 ----/\/\/\/\/---|   TR BC546B
;                       \
;                        |
; 0V  -------------------+--------
; 
; 
; +12V -----|>----+----[78L05]----+-----------------> 5 Volts
;           100F =       |       = 100 nF + 220F
;   0V -----------+-------+-------+-----------------
; 
; 
; Principe de dtection :
; -----------------------
; Longueur minimum d'un wagon HO = 10 cm.
; Vitesse maxi = 400 km/h = 120 cm/secondes sur la maquette HO.
; Wagon de 10 cm => Temps minimum de passage devant le capteur => 78 ms = Dure maximum de test pour activer le signal en sortie.
; 
; Pour ne pas tre pertub par les lampes alimentes en 50 Hz, on prend une dure minimum de test de passage = 
; 3 alternances  1/50 Hz  = 3*20 msec = 60 msec, soit 20 tests de 3 msec.
; 
; On part donc sur une dure de test de 60 msec, pour activer le signal en sortie.
; 
; Toutes les 3 millisecondes, on active la led infrarouge pendant 1,5 millisecondes.
; Le phototransistor met plus d'une milliseconde  ragir.
; 
; 12F675 : Courant max = 125 mA, courant max par E/S = 20 mA.
; Consommation du montage = 5 mA.
;          +---------+         +---------+         +---------+         +--------
; GP5 =    |   ON    | <1,5ms> |   ON    |         |   ON    |         |  ON
; ---------+ <1,5ms> +---------+         +---------+         +---------+
;         |         |         |         |         |         |         |  
;         v         v         v         v         v         v         v    
; GP2 = mesure2   mesure1
; 
; Si on a aucun obstacle (compteur=0, 10 fois de suite) ou si on a un obstacle en continu (compteur=25, 10 fois de suite), le montage passe en mode conomie de courant.
; Consommation en mode conome 5 mA.
; Si cpt_obs == 0,  nb_eco = nb_eco + 1
;          +---------+                                                 +--------
; GP5 =    |   ON    |                                                 |  ON
; ---------+         +-------------------------------------------------+
;           
; Mesures et tests raliss :
; ---------------------------
;   Si mesure2 > mesure1 + Sensibilit : Alors prsence obstacle, cpt_obs = cpt_obs + 1.
;                                        Valeur maximum = 25.
;   Si mesure2 < mesure1 + Sensibilit : Alors pas d'obstacle, cpt_obs = cpt_obs - 2.
;                                        Valeur minimum = 0.
; 
;   Si cpt_obs >= 20 => Armement de la temporisation de sortie.
;                       pot_tempo = pot_tempo_init.
;                    
;   Si pot_tempo > 0,  on active la sortie.
;   Si pot_tempo = 0,  on dsactive la sortie.
; 
;   Toutes les 0,1 secondes, on dcremente "pot_tempo", jusqu' "0".
;
; 
; << Programme principal >>
;
;     Dclaration des constantes.
;     cpt_obs_max =   25 ; Valeur maximum de comptage.
;     cpt_obs_det =   20 ; Seuil de dtection.
;     cpt_obs_inc =    1 ; Si prsence d'obstacle.
;     cpt_obs_dec =    3 ; Si abscence d'obstacle.
;     cpt_sortie2_init = 33 ; Pour une boucle de 0,1 sec (0,1/0,003).
;     max_eco = 4        ; Nb de pas sauts en mode conome.
;
;     Dclaration des variables.
;     etat_led           ; Led IR (0=Off, 1=On).
;     cpt_obs            ; Compteur de dtection d'obstacle.
;     cpt_sortie2        ; Compteur de boucle pour obtenir 0,1 sec.
;     mode_eco           ; Mode mode_eco (0 => mode normal,  1=> mode mode_eco).
;     pot_sensi          ; Sensibilit de la dtection, = Valeur en retour de table.
;     pot_tempo          ; Compteur de temporisation de la sortie *0,1 sec. Valeur de la sortie (0=Off, 1=On si cpt_t_out>0).
;     pot_tempo_init     ; Valeur d'initialisation du compteur de temporisation de la sortie *0,1 sec.
;     mesure2            ; Valeur de l'entre analogique GP2 (lecture phototransistor, led IR teinte).
;     mesure1            ; Valeur de l'entre analogique GP2 (lecture phototransistor, led IR allume).
;     nb_eco             ; Nombre le non detection en cours.
; 	  cpt_eco			 ; Nombre de pas  sauter
;
;     Initialisation des ports et des variables.
;     etat_led = 0
;     cpt_obs = 0
;     cpt_sortie2 = cpt_sortie_2_init
;     mode_eco = 0
;     pot_sensi = 77
;     pot_tempo = 0
;     pot_tempo_init = 10
;     mesure2 = 128
;     mesure1 = 128
;     nb_eco = 0
;
;     Lancement CNA sur GP0.
;     Initialisation pour une interruption toutes les 1,5 millisecondes.
;     Boucle infinie ici <--->
; 
; 
; << Routine d'interruption >>
;     On passe dans cette routine toutes les 1,5 msec.
;     Si mode_eco > 0 ?
;       mode_eco = mode_eco - 1
;       Fin de la routine pour viter d'allumer la LED IR trop souvent, en absence de dtection d'obstacle.
;       Permet de diviser la consommation du montage par trois (10 mA -> 3 mA).
;       Return
; 
;     Si etat_led = 0  ? => La led IR est teinte !
;       Lecture CAN GP0 => pot_sensi = Potentiomtre de sensibilit de la dtection.
;       Lancement CAN sur GP2 (Lecture phototransistor, led IR teinte).
;       Temporisation attente fin CAN sur GP2.
;       Lecture CAN sur GP2 => mesure2 (Lecture phototransistor, led IR teinte).
;       Lancement CAN sur GP1  (Potentiomtre de dure temporisation) et lecture plus tard.
; 
;     Si etat_led = 1 ? => La led infrarouge est allume !
;       Lecture CAN GP1 => pot_tempo_init = Potentiomtre de dure du signal en sortie, en fois 0,1 sec.
;       Lancement CAN sur GP2 (Lecture phototransistor, led IR allume).
;       Temporisation attente fin CAN sur GP2.
;       Lecture CAN sur GP2 => mesure1 (Lecture phototransistor, led IR allume).
;       Lancement CAN sur GP0  (Potentiomtre de sensibilit de la dtection) et lecture plus tard.
;       Si mesure1 + pot_sensi =< mesure2 ? => Alors prsence d'obstacle !
;         cpt_obs = cpt_obs + cpt_obs_inc  Compteur de prsence d'obstacle.
;         Si cpt_obs >= cpt_obs_det ? => Dtection d'obstacle ! => Activation de la sortie !
;           pot_tempo = pot_tempo_init  => Dure du signal en sortie en fois 0,1 sec.
;           Si cpt_obs >= cpt_obs_max => cpt_obs = cpt_obs_max = valeur maximum.
;       Sinon pas de prsence d'obstacle.
;         cpt_obs = cpt_obs - cpt_obs_dec  Dcompte en absence d'obstacle.
;         avec cpt_obs minimum = 0.
; 	Si cpt_obs = 0  ?
;           C'est au moins la deuxime fois que l'on n'a pas d'obstacle. 
;           mode_eco = 4  On n'a pas detect d'obstacle. On passe alors en mode conomie d'nergie. 
;                        On allume la led IR que toute les toutes les 12 msec, pour rduire la consommation de courant.
; 
;       cpt_019 = cpt_019 - 1
;       Si cpt_019 < 0  ?   
;         cpt_019 = cpt_019_init   Avec 126*1,5ms = 0,1 sec, on passe ici toutes les 0,1 secondes.
;         Si pot_tempo = 0 ?
;           Sortie GP5 = 0
;         Sinon
;           Sortie GP5 = 1
;           pot_tempo = pot_tempo - 1.
; 
;     Inverse le bit (0) de etat_led = (0/1 alternativement)
;     Si etat_led = 1 ?
;       GP4 = ON, pour allumer la LED IR
;     Sinon
;       GP4 = ON, pour teindre la LED IR
;     Return
; 
;----------------------------------------------------------------------------------------------------------------
;                            12F675
;          +5 Volts    VDD  [1    8]  VSS  Masse
;  Sortie   Output     GP5  [2    7]  GP0 AN0 = Potentiomtre sensibilit de la dtection
;  LED IR   Output     GP4  [3    6]  GP1 AN1 = Potentiomtre dure signal en sortie
;  Ref/Dir  Input      GP3  [4    5]  GP2 AN2 = Entre phototransistor
;----------------------------------------------------------------------------------------------------------------

	LIST      	p=PIC12F675     ; Dfinition du processeur.
	#INCLUDE 	<p12F675.inc> 	; Dfinitions des constantes.
	ERRORLEVEL 	-302
	__CONFIG  _CPD_OFF & _BODEN_OFF  & _CP_OFF & _WDT_OFF & _PWRTE_ON & _MCLRE_OFF & _INTRC_OSC_NOCLKOUT 
					; Oscillateur interne 4 MHz.

; Dclaration des constantes :
cpt_obs_max		 equ .25	; Valeur maximum du comptage de top IR. On bloque  cette valeur. (cpt_obs_det < cpt_obs_max < 255).
cpt_obs_inc		 equ  .1	; En cas de dtection IR, on incrmente le compteur de top IR de cette valeur. (1  5).
cpt_obs_dec		 equ  .3	; En cas de non dtection IR, on dcrmente le compteur de top IR de cette valeur. (1  10).
cpt_obs_det		 equ .20	; Seuil de dtection pour activer la sortie. Si le compteur de top IR dpasse cette valeur, on active la sortie.
							; cpt_obs_det < cpt_obs_max.
cpt_eco_max		 equ  .5	; Nombre de cycle sans activation de la led IR en mode conome 5 = 1/3 de cycle.
cpt_sortie2_init equ .66	; Pour une boucle de dcompte de 0,1 sec (66=0,1/0,0015).
max_eco			 equ .10	; Pour passer en mode economique, nb de fois ou le compteur = 0 ou 25.

BANK0	macro
	bcf	STATUS , RP0	; Passer en banque0.
	endm
BANK1	macro
	bsf	STATUS , RP0	; Passer en banque1.
	endm

; Dfinition des ports utiliss
#define	GPIOled	 GPIO,4	; Sortie led IR.
#define	GPIOout	 GPIO,5	; Sortie signal dtection.
#define	GPIOredi GPIO,3	; Mode Rflexion ou Direct.

; Dclaration des variables :
	CBLOCK 0x020  		; Dbut de la zone des variables.
	etat_led			; Pour basculer de squence, led IR : 0 = teinte / 1 = allume.
	pot_sensi			; Lecture POT0, pour rgler la sensibilit du dtecteur.
 	pot_tempo			; Lecture POT1, pour rgler la dure de temporisation en sortie.
	cpt_sortie			; Compteur de temporisation de la sortie * 0,1 sec. Si > 0, on active la sortie.
	cpt_sortie2			; Compteur pour avoir des tops gale  0,1 secondes.
	cpt_obs				; Pour compter en temps rel le nombre de dtections d'obstacle positifs  la suite.
	mesure1				; Valeur de l'entre analogique GP2 (lecture du phototransistor, led IR allume).
	mesure2				; Valeur de l'entre analogique GP2 (lecture du phototransistor, led IR teinte).
	mode_eco			; Si == 1 => on passe en mode conome pour allumer moins souvent la led IR.
	nb_eco				; Si cpt_obs = 0 ou 25, on incrmente ce compteur jusqu' max_eco avant de faire mode_eco = 1 !
	cpt_eco				; Utilis si mode_eco == 1. => Pour activer la led une fois / cpt_eco_max. 
	temp				; Registre temporaire.
	redi_sav			; Sauvegarde de la dernire position de l'interrupteur
	w_temp				; Sauvegarde des registres dans la partie interruption.
	status_temp			; Sauvegarde des registres dans la partie interruption.
	ENDC				; Fin de la zone des variables    
;
; Dmarrage sur reset
;**********************************************************************
	org		0x2100 	  			 ; Initialize EEPROM Data
	dt 		"Detecteur infrarouge par reflexion V1 "	; Texte visible dans le '.hex'.
	dt		"Le 28/04/2021 "
	dt		"http://www.la-tour.info/uts/uts_index.html"
	org 	0x000 			; Adresse aprs reset.
  	goto    Init
	org 	0x004			; Adresse lors d'interruptions.
	goto	Interrupt		
;*************************************************************************************
	dt		"__DETECTEUR_INFRAROUGE_PAR_REFLEXION_V1__"	; Texte visible dans le '.hex'.
	dt		"__Le 28/04/2021___"
	dt		"__http://www.la-tour.info/uts/uts_index.html__"
;***************************************************************************************************************
;
; Initialisation du programme
;**********************************************************************
Init
; initialisation du fonctionnement interne.
	BANK1
	call    3FFH			; On utilise l'horloge interne  4 Mhz.
	movwf   OSCCAL         	; Mise  jour du registre de calibration de l'horloge interne.
	clrf	INTCON			; Pas d'interruption pour le moment.
	movlw	B'10000010'		; Bit 7 : 1 = Pas de rsistance de pull-up. Bit 2-0 : Prescaler TMR0 = "010" => TMR0_in = 4Mhz/4/8.
	movwf	OPTION_REG		; TMR0_in =  125 Khz => TMR0 au pas de 8 S donc 256*8S => Comptage de 2,048 msec maxi.
	clrf	IOC				; IOC   : Pas d'interruption sur changement d'tat des entres. 
	clrf	PIE1			; PIE1  : Pas d'interruption sur changement d'tat A/D, EEWrite, Comparateurs ou Dbordement TMR1.
	clrf	VRCON			; VRCON : Rfrence de tension.
	movlw	B'00010111'		; x001 = Fosc/8 (TAD=2s), entres analogiques = AN2, AN1 et AN0 => "0111".
	movwf	ANSEL			; Frq interne = 4 Mhz, CAN de 2 S = Fosc/8 => "0001".

	BANK0
	clrf  	GPIO			; Raz sorties du port GPIO.
	BANK1
	movlw	B'00001111'		; Sorties GPIO basse impdance ="0" pour GP4 et GP5, haute impdance ="1" pour GP0, GP1 et GP2.
	movwf	TRISIO

	BANK0
	clrf	T1CON			; T1CON : Timer1.
	movlw	b'00000111'		; Comparateurs  l'arrt.
	movwf	CMCON
	movlw	b'00000001'		; Entres analogiques, rsultat sur 8 bits de poids forts, rfrence interne=Vcc, CAN en marche sur AN0.
	movwf	ADCON0

; initialisation des variables.
	clrf	etat_led		; Pour basculer de squence, led IR : 0 = teinte / 1 = allume.
	clrf	cpt_obs			; Pour compter en temps rel le nombre de dtections d'obstacle positifs  la suite.
	clrf	mode_eco		; 0 = Mode normal, si == 1, on passe en mode conome pour allumer la led IR moins souvent.
	clrf	nb_eco			; 
	clrf	cpt_sortie		;
	movlw	cpt_sortie2_init	; Nombre de boucles pour obtenir 0,1 secondes,  partir de 1,5 msec.
	movwf	cpt_sortie2
	movlw	cpt_eco_max
	movwf	cpt_eco
	movlw	.77
	movwf	pot_sensi		; pot_sensi = 44, valeur max par dfaut avant de lire POT0.
	movlw	.128
	movwf	pot_tempo		; Pour la temporisation moyenne du signal en sortie.
	movwf	mesure2			; Mesures moyenne par dfaut avant de lire l'entre analogique du phototransistor.
	movwf	mesure1
	movlw	.20
	movwf	temp
	decfsz	temp,f
	goto	$-1				; Petite tempo aprs la slection de l'entre CAN.
	bsf 	ADCON0,GO_DONE	; Lance la mesure sur l'entre analogique AN0, pour la lire plus tard.
	movlw   .69				; Interruption toutes les 1,5 msec => (186*8S) => 255-69 = 186.
	movwf   TMR0			; Interruption  partir de maintenant.
	bcf     INTCON,T0IF		; Remise  0 du drapeau de dpassement du Timer0 pour la prochaine interruption.
	bsf     INTCON,T0IE		; L'initialisation est finie, on autorise l'interruption sur TRM0.
	bsf     INTCON,GIE		; On autorise globalement toutes les interruptions.
	clrf	redi_sav		;
	btfsc	GPIOredi		; Mode Rflexion ou mode Direct.
	bsf		redi_sav,0		; 0 ou 1.

Boucle		
	nop
	nop
	nop
	goto 	Boucle			; En attente d'interruption.

;**********************************************************************
; << Routine d'interruption >>
; On passe dans cette routine toutes les 1,5 msec.
Interrupt
	bcf     INTCON,GIE		; Inhibe toutes les interruptions.
	movwf   w_temp  		; Sauve le registre W.
	swapf	STATUS,w		; Swap status avec rsultat dans w.
	movwf	status_temp		; Sauver le STATUS. Pas besoin de passer en BANK0, car on n'utilise ni BANK1 ni BANK2 dans la boucle principale.
	movlw   .69				; Interruption toutes les 1,507 msec => (186*8S) => 255-69 = 186.
	movwf   TMR0
	bcf     INTCON,T0IF		; Remise  0 du drapeau de dpassement du Timer0 pour la prochaine interruption.

Eco
; Si mode_eco == 0 ?
;   mode_eco = mode_eco - 1
;   Fin de la routine pour viter d'allumer la LED IR trop souvent, en absence de dtection d'obstacle.
;   Permet de diviser la consommation du montage par trois (10 mA -> 3 mA).
;   Return
	
	btfsc	etat_led,0
	goto	not_eco			
	movf	mode_eco,w		; On ne passe ici, que si la led est dj teinte.
	btfsc   STATUS,Z
	goto	not_eco
	decfsz	cpt_eco,f		; cpt_eco = cpt_eco - 1.
	goto	sortie_0		; Fin, mais on traite quand mme le signal de sortie.
	movlw	cpt_eco_max
	movwf	cpt_eco

not_eco
; Si etat_led = 0  ? => La led IR est teinte !
;   Lecture CAN GP0 => pot_sensi = Potentiomtre de sensibilit de la dtection.
;   Lancement CAN sur GP2 (Lecture phototransistor, led IR teinte).
;   Temporisation attente fin CAN sur GP2.
;   Lecture CAN sur GP2 => mesure2 (Lecture phototransistor, led IR teinte).
;   Lancement CAN sur GP1  (Potentiomtre de dure temporisation) et lecture plus tard.

	btfsc	etat_led,0
	goto	led_allumee		; etat_led == 1 => Goto "Led_allumee".
led_eteinte	
	btfsc	ADCON0,GO		; Conversion AN0 finie ?
	goto	$-1
 	movlw	0x03
	movwf	PCLATH
	swapf	ADRESH,w		; Transfre le rsultat AN0 dans w.
	andlw	0x0F			; W = 0  15.
	call	sensi_table		; Table intermdiaire, pour une sensibilit non linaire.
	movwf	pot_sensi		; pot_sensi = 1  77.
	bsf 	ADCON0,CHS1
	bcf 	ADCON0,CHS0		; 10 = AN2 = GP2 = Entre Phototransistor.	
	movlw	.10
	movwf	temp
	decfsz	temp,f
	goto	$-1				; Petite tempo obligatoire aprs la slection de l'entre CAN.
	bsf 	ADCON0,1		; Lance la mesure sur l'entre analogique AN2 = Entre phototransistor IR.
	btfsc	ADCON0,GO		; Conversion AN2 finie ?
	goto	$-1
	movf	ADRESH,w		; Transfre le rsultat dans w.
	movwf	mesure2			; mesure2 = 0  255 = Mesure led teinte.
	bsf		etat_led,0		; Changement d'tat de la led IR.
	bsf		GPIOled			; Allume la led : Sortie = +5 Volts.
	bcf 	ADCON0,CHS1
	bsf 	ADCON0,CHS0		; 01 = AN1 = GP1 = Entre potentiomtre POT1 = Dure signal en sortie.
	movlw	.10
	movwf	temp
	decfsz	temp,f
	goto	$-1				; Petite tempo obligatoire aprs la slection de l'entre CAN.
	bsf 	ADCON0,1		; Lance la mesure sur l'entre analogique AN1 pour une lecture plus tard.
	goto	sortie_0			; Fin

led_allumee
; Si etat_led = 1 ? => La led infrarouge est allume !
;   Lecture CAN GP1 => pot_tempo_init = Potentiomtre de dure du signal en sortie, en fois 0,1 sec.
;   Lancement CAN sur GP2 (Lecture phototransistor, led IR allume).
;   Temporisation attente fin CAN sur GP2.
;   Lecture CAN sur GP2 => mesure1 (Lecture phototransistor, led IR allume).
;   Lancement CAN sur GP0  (Potentiomtre de sensibilit de la dtection) et lecture plus tard.

	btfsc	ADCON0,GO		; Conversion AN1 finie ?
	goto	$-1
; Entre GP1 = AN1 = 0-5 Volts = Rglage de la dure du signal en sortie, de 0  25 secondes.
	movf	ADRESH,w		; Transfre le rsultat dans w.
	movwf	pot_tempo		; cpt_t_tout = 0  255.
	addlw	-2
	btfss	STATUS,C
	incf	pot_tempo,f		; 2 minimum.
	btfss	STATUS,C
	incf	pot_tempo,f		; 2 minimum, sinon on a des crneaux en sortie si pot_tempo < 2.
	
	bsf 	ADCON0,CHS1
	bcf 	ADCON0,CHS0		; 10 = AN2 = GP2 = Entre Phototransistor.	
	movlw	.10
	movwf	temp
	decfsz	temp,f
	goto	$-1				; Petite tempo obligatoire aprs la slection de l'entre CAN.
	bsf 	ADCON0,1		; Lance la mesure sur l'entre analogique AN2
	btfsc	ADCON0,GO		; Conversion AN2 finie ?
	goto	$-1
	movf	ADRESH,w		; Transfre le rsultat dans w.
	movwf	mesure1			; mesure1 = 0  255 = Mesure led allume.
	bcf		etat_led,0		; Changement d'tat de la led IR.
	bcf		GPIOled			; Eteint la led : Sortie = 0 Volt.
	bcf 	ADCON0,CHS1
	bcf 	ADCON0,CHS0		; 00 = AN0 = GP0 = Entre potentiomtre rglage de la sensibilit.
	movlw	.10
	movwf	temp
	decfsz	temp,f
	goto	$-1				; Petite tempo obligatoire aprs la slection de l'entre CAN.
	bsf 	ADCON0,1		; Lance la mesure sur l'entre analogique AN0 pour une lecture plus tard.

;-------------------------------------------------------------------------------------------------
; Si mesure1 + pot_sensi =< mesure2 ? => Alors prsence d'obstacle !
;   cpt_obs = cpt_obs + cpt_obs_inc  Compteur de prsence d'obstacle.
;   Si cpt_obs >= cpt_obs_det ? => Dtection d'obstacle ! => Activation de la sortie !
;     pot_tempo = pot_tempo_init  => Dure du signal en sortie en fois 0,1 sec.
;     Si cpt_obs >= cpt_obs_max => cpt_obs = cpt_obs_max = valeur maximum.
;   Sinon pas de prsence d'obstacle.
;     cpt_obs = cpt_obs - cpt_obs_dec  Dcompte en absence d'obstacle.
;     avec cpt_obs minimum = 0.
;   Si cpt_obs = 0  ?
;      C'est au moins la deuxime fois que l'on n'a pas d'obstacle. 
;      mode_eco = 4  On n'a pas detect d'obstacle. On passe alors en mode conomie d'nergie. 
;                   On allume la led IR que toute les toutes les 12 msec, pour rduire la consommation de courant.

compare_1					; Si l'on bouge l'interrupteur, on fait un raz de la sortie.
	btfss	GPIOredi		; Mode Rflexion ou Mode Direct ?
	goto	compare_1a
	btfss	redi_sav,0		; GPIOredi == 1!
	clrf	cpt_sortie		; GPIOredi == 1 et redi_save= 0 => cpt_sortie = 0!
	btfss	redi_sav,0		; GPIOredi == 1!
	clrf	cpt_obs
	bsf		redi_sav,0		; 
	goto	compare_1b
compare_1a
	btfsc	redi_sav,0		; GPIOredi == 0!
	clrf	cpt_sortie		; GPIOredi == 0 et redi_save= 1 => cpt_sortie = 0!
	btfsc	redi_sav,0		; GPIOredi == 0!
	clrf	cpt_obs		
	bcf		redi_sav,0		; 

compare_1b
	movf	mesure1,w
	subwf	mesure2,w		; mesure2 == mesure1 ?
	btfss	STATUS,Z		
	goto	compare_2
; 	mesure2 == mesure1 ! => On ne fait rien.
	goto	compare_fin

compare_2
	movf	mesure1,w
	subwf	mesure2,w		; mesure1 > mesure2 ?
	btfsc	STATUS,C		
	goto	compare_3
; 	mesure1 > mesure2 ! => On ne fait rien.
	goto	compare_fin

compare_3
	movwf	temp
	movf	pot_sensi,w		; mesure1 < mesure2 et w=(mesure2 - mesure1) >  pot_sensi ?
	subwf	temp,w
	btfss	STATUS,C		
	goto	compare_4
;	mesure2 - mesure1 > pot_sensi => cpt_obs++ = Obstacle en face.
;;;	bsf		GPIOout			; On active la sortie signal. ; Uniquement pour DEBUG AVIRER
	btfss	GPIOredi		; Mode Rflexion ou mode Direct.
	goto	compare_4b		; Si mode "Direct".

compare_3b
	movf	cpt_obs,w
	addlw	cpt_obs_inc
	movwf	cpt_obs
	addlw	-cpt_obs_max
	movlw	cpt_obs_max		; On ne peut pas dpasser cpt_obs_max !
	btfsc	STATUS,C		
	movwf	cpt_obs
	goto	compare_fin

compare_4
	btfss	GPIOredi		; Mode Rflexion ou mode Direct.
	goto	compare_3b		; Si mode "Direct".
compare_4b
;	mesure2 - mesure1 < pot_sensi => cpt_obs-- = Pas d'obstable en vue.
;;;	bcf		GPIOout			; On active la sortie signal. ; Uniquement pour DEBUG AVIRER
	movf	cpt_obs,w
	addlw	-cpt_obs_dec
	btfss	STATUS,C
	movlw	.0				; On ne peut pas descendre au dessus de 0 !
	movwf	cpt_obs

compare_fin
; Au final : 0 =< cpt_obs =< cpt_obs_max
;-------------------------------------------------------------------------------------------------

	movf	cpt_obs,w
	sublw	cpt_obs_max
	btfsc	STATUS,Z	
	incfsz	nb_eco,f		; Si cpt_obs==cpt_obs_max => nb_eco = nb_eco + 1.
	nop
	btfsc	STATUS,Z	
	goto	compare_10
	movf	cpt_obs,w
	btfsc	STATUS,Z	
	incfsz	nb_eco,f		; Si cpt_obs==0 => nb_eco = nb_eco + 1.
	nop
	btfsc	STATUS,Z	
	goto	compare_10
	clrf	nb_eco			; Si cpt_obs>0 et cpt_obs<cpt_obs_max => nb_eco = 0 On est en phase active de comptage. 

compare_10
	movf	nb_eco,w
	addlw	-max_eco-1
	btfss	STATUS,C
	goto	compare_11
	decf	nb_eco,f		; Pour ne pas dpasser max_eco.
	movlw	1
	movwf	mode_eco
	goto	compare_12		; mode_eco = ON.
		
compare_11
	clrf	mode_eco		; mode_eco = OFF.

compare_12

;-------------------------------------------------------------------------------------------------
sortie_0
	movf	pot_tempo,w		; Si l'on rduit la tempo au potentiomtre, 
	subwf	cpt_sortie,w	; on rduit aussi le compteur "cpt_sortie" courant.
	btfsc	STATUS,Z		; cpt_sortie ne peut pas tre > pot_tempo (2  255) 255 = 25 secondes.
	goto	sortie_1
	btfss	STATUS,C		
	goto	sortie_1
	movf	pot_tempo,w		
	movwf	cpt_sortie		; On r-arme la sortie pour X secondes max.

sortie_1
; Si cpt_obs > cpt_obs_det => Confirmation d'un obstacle. 
; On passe ici toutes les 1,3 msec.
	movf	cpt_obs,w
	sublw	cpt_obs_det		; cpt_obs > cpt_obs_det ?
	btfsc	STATUS,C		
	goto	sortie_2
; 	cpt_obs < cpt_obs_det ! => On ne fait rien.
	movf	pot_tempo,w		
	movwf	cpt_sortie		; On r-arme la sortie pour X secondes.
	bsf		GPIOout			; On active la sortie signal.

sortie_2
	decfsz	cpt_sortie2,f	; Compteur rapide pour obtenir 0,1 sec.
	goto	sortie_fin
; on passe ici toutes les 100 ms.
	movlw	cpt_sortie2_init
	movwf	cpt_sortie2
	movf	cpt_sortie,w
	btfss	STATUS,Z
	decf	cpt_sortie,f	; Dcrmente cpt_sortie jusqu' 0.
	btfsc	STATUS,Z
	bcf		GPIOout			; Raz sortie signal.

sortie_fin

;-------------------------------------------------------------------------------------------------
Fin_int	
	swapf	status_temp,w	; swap ancien status, rsultat dans w
	movwf   STATUS			; restaurer status
	swapf   w_temp,f		; Inversion L et H de l'ancien W sans modifier Z
	swapf   w_temp,w  		; Rinversion de L et H dans W. W restaur sans modifier status
	bsf     INTCON,GIE		; Autorise toutes les interruptions
	retfie  				; Retour d'interruption.
;
;
;-------------------------------------------------------------------------------------------------
;-------------------------------------------------------------------------------------------------
;-------------------------------------------------------------------------------------------------
; La plage de sensibilit du phototransistor IR est dcoupe en 16 pas : (0-255)/16.
; Table de conversion non linaire.
;  Entre : w de 0  15 = AN0/16 = POT0/16.
;  Sortie : de 2 = trs sensible,  44 = peu sensible.
	ORG 0x300				; Adresse des tableaux en fin de mmoire (moins de 256 octets)
sensi_table ;
	addwf	PCL,f			; Ajouter w  PCL
	retlw	.2	;    0
	retlw	.3	;    1
	retlw	.4	;    2
	retlw	.5	;    3
	retlw	.6	;    4
	retlw	.8	;    5
	retlw	.10	;    6
	retlw	.13	;    7
	retlw	.16	;    8
	retlw	.20	;    9
	retlw	.25	;   10
	retlw	.32	;   11
	retlw	.40	;   12
	retlw	.50	;   13
	retlw	.62	;   14
	retlw	.77	;   15		; Fin de table 

	END